/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import com.apple.eawt.CocoaComponent;

public abstract class DMWebKitView extends CocoaComponent{

	public static final int storeJavaObject         = 1;
	public static final int loadURL			= 2;
	public static final int resizeFrame		= 3;
        public static final int stopLoading             = 4;
        public static final int goBack                  = 5;
        public static final int goForward               = 6;
        public static final int updateCursor            = 7;
        public static final int runJS                   = 8;
        public static final int textLarger              = 9;
        public static final int textSmaller             = 10;
        public static final int search                  = 11;
        public static final int dispose                 = 12;
        public static final int ordercallback           = 13;
        public static final int loadURLFromString       = 14;
        public static final int reload                  = 15;
        
        public static final int INIT_WIDTH              = 500;
        public static final int INIT_HEIGHT             = 420;

        
        public static final int WEB_VIEW_POLICY_NONE                = 0;
        public static final int WEB_VIEW_POLICY_IN_OLD_WINDOW       = 1;
        public static final int WEB_VIEW_POLICY_IN_NEW_WINDOW       = 2;
        
        
        protected boolean needResize = false;
    Vector  hyperlinkListeners = new Vector();
    java.net.URL  lastURL = null;
    private String HTMLSource = null;
    abstract public int createNSView();
 
    static WebKitElementInfo webKitElementInfo = null;   
    
    public static int testCount = 0;
    
    protected int newWebViewCreatingPolicy = WEB_VIEW_POLICY_IN_OLD_WINDOW;

    static {
        loadLibrary();
    }
    
    private static void loadLibrary(){
        org.dm.webkit.installer.WebKitInstaller webkitInstaller = new org.dm.webkit.installer.WebKitInstaller();
        String libPath = webkitInstaller.getJNILibLibraryPath();
        java.io.File libFile = (libPath == null)?null:new java.io.File(libPath);
        try {
            if(libFile == null || !libFile.exists()){
                System.loadLibrary("webkitsample");
            }else{
                System.load(libFile.getCanonicalPath());
            }
        }
        catch(Throwable t) {
            t.printStackTrace();
        }
    }
    
    public DMWebKitView(){
    	super();
/*
        addComponentListener(new ComponentAdapter(){
            public void componentResized(ComponentEvent e) {
                javax.swing.SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        System.out.println("run componentResized");
                        //sendMessage(DMWebKitView.resizeFrame,DMWebKitView.this);
                        DMWebKitView.this.repaint();
                        DMWebKitView.this.getParent().doLayout();
                    }
                });
            }
        });
*/
    }
    
    public int getNewWebViewCreatingPolicy(){
        return newWebViewCreatingPolicy;
    }
 
    public void setNewWebViewCreatingPolicy(int newWebViewCreatingPolicy){
        this.newWebViewCreatingPolicy = newWebViewCreatingPolicy;
        if(this.newWebViewCreatingPolicy < WEB_VIEW_POLICY_NONE || 
           this.newWebViewCreatingPolicy > WEB_VIEW_POLICY_IN_NEW_WINDOW){
            this.newWebViewCreatingPolicy = WEB_VIEW_POLICY_IN_OLD_WINDOW;
        }
    }
    
    public void addHyperlinkListener(HyperlinkListener l){
        if(l == null) return;
        if(!hyperlinkListeners.contains(l)){
            hyperlinkListeners.addElement(l);
        }
    }

    public void removeHyperlinkListener(HyperlinkListener l){
        if(l == null) return;
        if(hyperlinkListeners.contains(l)){
            hyperlinkListeners.removeElement(l);
        }
    }
    
    void notifyHyperlinkListeners(HyperlinkEvent e){
        if(hyperlinkListeners == null || hyperlinkListeners.size() < 1) return;
        final HyperlinkEvent evt = e;
        javax.swing.SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                for(int i = 0; i < hyperlinkListeners.size(); i++){
                    HyperlinkListener l = (HyperlinkListener)hyperlinkListeners.elementAt(i);
                    try{
                        l.hyperlinkUpdate(evt);
                    }catch(Throwable t){
                        t.printStackTrace();
                    }
                }
            }
        });
    }
    
    
    public Object getWebKitElementInfo(){
        if(webKitElementInfo == null) webKitElementInfo = new WebKitElementInfo();
        return webKitElementInfo;
    }
    public void addNotify(){
    	super.addNotify();
    	sendMessage(DMWebKitView.storeJavaObject,this);
//        synchronized(notifyObject){
//            notifyObject.notifyAll();
//        }
//        System.out.println("notifyAll "+Thread.currentThread());
//    	addMouseListener(new MouseAdapter(){
//           public void mouseReleased(MouseEvent e) {
//				requestFocusInWindow();
//            }
//    	});
    }

    public void refresh(){
        sendMessage(resizeFrame,this);

 Container c = getParent();
        if(c != null) c.repaint();
        if(needResize){
            Dimension cd = getSize();
            //System.out.println("cd "+cd);
            cd.width++;
            cd.height++;
            setSize(cd);
            cd.width--;
            cd.height--;
            setSize(cd);
            needResize = false;
        }
 
    }

    public void sendMessage(int messageID){
        sendMessage(messageID,null);
    }
    
    public String runJS(String js){
        StringBuffer sb = new StringBuffer();
        Object []args = {js,sb};
        sendMessage(runJS,args);
        return sb.toString();
    }

    public void reload(){
        sendMessage(reload);
    }

    public void makeTextLarger(){
        sendMessage(textLarger);
    }

    public void makeTextSmaller(){
        sendMessage(textSmaller);
    }

    protected void reRenderDocument(){//workaround!
        makeTextLarger();
        makeTextSmaller();
    }

    public void validate(){
        super.validate();
//        reRenderDocument();
    }
    
    public void dispose(){
        sendMessage(dispose);
    }

    protected void orderCallback(){
        sendMessage(ordercallback);
    }

    protected void callback(Runnable runnable){
        if(runnable != null) runnable.run();
    }
/**
 * @return true if you need to pop-up Alert dialog otherwise false
 */
    public boolean runJavaScriptAlertPanelWithMessage(String message){
        System.out.println("Java runJavaScriptAlertPanelWithMessage "+message);
        return true;
    }

    public String getHTMLSource(){
        return HTMLSource;
    }
    
    private void clearHTMLSource(){
        HTMLSource = null;
    }
    
    protected void setHTMLSource(String str){
        clearHTMLSource();
        if(str != null) HTMLSource = new String(str);
    }

    public void invokeFromWebKitThread(Runnable runnable){
        sendMessage(ordercallback,runnable);
    }

    protected void finalize() throws Throwable{
        dispose();
        super.finalize();
    }

/**
 * Starts the search from the current selection.  Will search across all frames.
 * method highlights the string if it is found.
 * @param stringToFind <code>String</code> The string to search for.
 * @param forward <code>boolean</code>. <code>true</code> to search forward, <code>false</code> to seach backwards.
 * @param caseFlag <code>boolean</code>. <code>true</code> to for case-sensitive search,<br>
 *  <code>false</code> for case-insensitive search.
 * @param wrap <code>boolean</code>. 
 * @return <code>true</code> if found, <code>false</code> if not found.
 */    

    public boolean searchForString(String stringToFind,boolean forward,boolean caseFlag, boolean wrap){
        Object []args = new Object[2];
        args[0] = stringToFind;
        args[1] = new boolean[4];
        ((boolean [])args[1])[0] = forward;
        ((boolean [])args[1])[1] = caseFlag;
        ((boolean [])args[1])[2] = wrap;
        ((boolean [])args[1])[3] = false;
        sendMessage(search,args);
        return ((boolean [])args[1])[3];
    }
    
    //callback
    public void finishURLLoading(){
        //System.out.println("JAVA finishURLLoading22");
        refresh();
        //System.out.println("JAVA HTMLSource\n "+getHTMLSource());
    }

    //callback
    public void startURLLoading(String urlStr){
        if(urlStr == null) return;
        clearHTMLSource();
        java.net.URL url = null;
        try{
            url = new java.net.URL(urlStr);
        }catch(Throwable t){
            url = null;
        }
        if(url != null) notifyHyperlinkListeners(new HyperlinkEvent(this,HyperlinkEvent.EventType.ACTIVATED,url));
     }
//callback
    public void mouseDidMoveOverElement(Object elementInfoObject){
    //HyperlinkListener
        if(!(elementInfoObject instanceof WebKitElementInfo)) return;
        String urlString = ((WebKitElementInfo)elementInfoObject).linkURLString;
        HyperlinkEvent.EventType eventType = null;
        java.net.URL url = null;
        if(lastURL != null && urlString == null){
            url = lastURL;
            eventType = HyperlinkEvent.EventType.EXITED;
            lastURL = null;
        }else if(lastURL == null && urlString != null){
            try{
                url = new java.net.URL(urlString);
            }catch(Throwable t){
                url = null;
            }
            lastURL = url;
            if(url != null){
                eventType = HyperlinkEvent.EventType.ENTERED;
            }
        }else{
            try{
                url = new java.net.URL(urlString);
            }catch(Throwable t){
                url = null;
            }
            if(url == null){
                url = lastURL;
                eventType = HyperlinkEvent.EventType.EXITED;
                lastURL = null;
            }else if(!url.equals(lastURL)){
                notifyHyperlinkListeners(new HyperlinkEvent(this,HyperlinkEvent.EventType.EXITED,lastURL));
                lastURL = url;
                eventType = HyperlinkEvent.EventType.ENTERED;
            }
        }
        if(url != null && eventType != null){
            notifyHyperlinkListeners(new HyperlinkEvent(this,eventType,url));
        }
//        WebKitElementInfo elementInfo = (WebKitElementInfo)elementInfoObject;
//        System.out.println("Java mouseDidMoveOverElement imageAltString "+elementInfo.imageAltString);
//        System.out.println("Java mouseDidMoveOverElement imageURLString "+elementInfo.imageURLString);
//        System.out.println("Java mouseDidMoveOverElement linkURLString "+elementInfo.linkURLString);
//        System.out.println("Java mouseDidMoveOverElement linkTitle "+elementInfo.linkTitle);
//        System.out.println("Java mouseDidMoveOverElement modifyer "+elementInfo.modifyer);
//        System.out.println("Java mouseDidMoveOverElement elementSelected "+elementInfo.elementSelected);
//        System.out.println("Java mouseDidMoveOverElement imageRect "+elementInfo.imageRect);
    }
}


