/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit.dom;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.DOMException;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Attr;
import org.w3c.dom.EntityReference;
import org.w3c.dom.Text;
import org.w3c.dom.Comment;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DocumentType;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DOMImplementation;

import org.dm.webkit.DMWebKitView;

public class DOMDocument extends AccessorHolder implements Document{
    DMWebKitView owner;
    NodeWrapper rootWrapper;
    NodeWrapper bodyWrapper;
    
    public DOMDocument(DMWebKitView owner){
        nodeAccessor = "window.document.documentElement";
        String bodyNodeAccessor = "window.document.body";
        this.owner = owner;
        short type = JSAccessors.getNodeType(owner,nodeAccessor);
        rootWrapper = NodeFactory.createWrapper(owner,nodeAccessor,type,this,null);
        type = JSAccessors.getNodeType(owner,bodyNodeAccessor);
        bodyWrapper = NodeFactory.createWrapper(owner,bodyNodeAccessor,type,this,null);
    }
    
    public NodeWrapper getRootNode(){return rootWrapper;}
    public ElementWrapper getBodyElement(){
        return (bodyWrapper instanceof ElementWrapper)?(ElementWrapper)bodyWrapper:null;
    }
    
    
    
    public Element getElementById(String elementId){
        throw new RuntimeException("getElementById isn't implemented");
    }

    public ProcessingInstruction createProcessingInstruction(String target, String data) throws DOMException{
        throw new RuntimeException("createProcessingInstruction isn't implemented");
    }
    
    public Attr createAttribute(String name) throws DOMException{
        throw new RuntimeException("createAttribute isn't implemented");
    }
    
    public Attr createAttributeNS(String namespaceURI, String qualifiedName) throws DOMException{
        throw new RuntimeException("createAttributeNS isn't implemented");
    }

    public Element createElementNS(String namespaceURI, String qualifiedName) throws DOMException{
        throw new RuntimeException("createElementNS isn't implemented");
    }

    public Node importNode(Node importedNode, boolean deep) throws DOMException{
        throw new RuntimeException("importNode isn't implemented");
    }

    public NodeList getElementsByTagName(String tagname){
        throw new RuntimeException("getElementsByTagName isn't implemented");
    }

    public NodeList getElementsByTagNameNS(String namespaceURI, String localName){
        throw new RuntimeException("getElementsByTagNameNS isn't implemented");
    }

    public EntityReference createEntityReference(String name) throws DOMException{
        throw new RuntimeException("createEntityReference isn't implemented");
    }
    
    public CDATASection createCDATASection(String data) throws DOMException{
        throw new RuntimeException("createCDATASection isn't implemented");
    }

    public Comment createComment(String data){
        throw new RuntimeException("createComment isn't implemented");
    }

    public Text createTextNode(String data){
        TextWrapper tw = (TextWrapper)NodeFactory.createWrapper(owner,null,Node.TEXT_NODE,this,null);
        tw.setNodeValue(data);
        return tw;
    }

    public DocumentFragment createDocumentFragment(){
        throw new RuntimeException("createDocumentFragment isn't implemented");
    }

    public Element createElement(String tagName) throws DOMException{
        ElementWrapper ew = (ElementWrapper)NodeFactory.createWrapper(owner,null,Node.ELEMENT_NODE,this,null);
        ew.tagName = tagName;
        return ew;
    }
    
    public Element getDocumentElement(){
        NodeWrapper nw = getRootNode();
        if(nw instanceof ElementWrapper) return (ElementWrapper)nw;
        return null;
    }
    
    public DOMImplementation getImplementation(){
        throw new RuntimeException("getImplementation isn't implemented");
    }
    
    public DocumentType getDoctype(){
        throw new RuntimeException("getDoctype isn't implemented");
    }
    
    
//Node methods    
    public String getNodeName(){
        throw new RuntimeException("getNodeName isn't implemented");
    }

    public String getNodeValue() throws DOMException{
        throw new RuntimeException("getNodeValue isn't implemented");
    }

    public void setNodeValue(String nodeValue) throws DOMException{
        throw new RuntimeException("setNodeValue isn't implemented");
    }

    public short getNodeType(){
        throw new RuntimeException("getNodeType isn't implemented");
    }

    public Node getParentNode(){
        return null;
    }

    public NodeList getChildNodes(){
        throw new RuntimeException("getChildNodes isn't implemented");
    }

    public Node getFirstChild(){
        throw new RuntimeException("getFirstChild isn't implemented");
    }

    public Node getLastChild(){
        throw new RuntimeException("getLastChild isn't implemented");
    }

    public Node getPreviousSibling(){
        return null;
    }

    public Node getNextSibling(){
        return null;
    }

    public NamedNodeMap getAttributes(){
        throw new RuntimeException("getAttributes isn't implemented");
    }

    public Document getOwnerDocument(){
        return null;
    }

    public Node insertBefore(Node newChild, Node refChild) throws DOMException{
        throw new RuntimeException("insertBefore isn't implemented");
    }

    public Node replaceChild(Node newChild, Node oldChild) throws DOMException{
        throw new RuntimeException("replaceChild isn't implemented");
    }

    public Node removeChild(Node oldChild) throws DOMException{
        throw new RuntimeException("removeChild isn't implemented");
    }

    public Node appendChild(Node newChild) throws DOMException{
        throw new RuntimeException("appendChild isn't implemented");
    }

    public boolean hasChildNodes(){
        throw new RuntimeException("hasChildNodes isn't implemented");
    }

    public Node cloneNode(boolean deep){
        throw new RuntimeException("cloneNode isn't implemented");
    }

    public void normalize(){
        throw new RuntimeException("normalize isn't implemented");
    }

    public boolean isSupported(String feature, String version){
        throw new RuntimeException("isSupported isn't implemented");
    }

    public String getNamespaceURI(){
        throw new RuntimeException("getNamespaceURI isn't implemented");
    }

    public String getPrefix(){
        throw new RuntimeException("getPrefix isn't implemented");
    }

    public void setPrefix(String prefix) throws DOMException{
        throw new RuntimeException("setPrefix isn't implemented");
    }

    public String getLocalName(){
        throw new RuntimeException("getLocalName isn't implemented");
    }

    public boolean hasAttributes(){
        throw new RuntimeException("hasAttributes isn't implemented");
    }

}

