/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit.dom;

import java.util.Vector;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.DOMException;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;

import org.dm.webkit.DMWebKitView;

public class ElementWrapper extends NodeWrapper implements Element{
AttrWrapper []attrs;
String tagName;
HashMap orphanAttributes;

    public ElementWrapper(DMWebKitView owner,String nodeAccessor,DOMDocument ownerDocument,Node parentNode){
        super(owner,nodeAccessor,ELEMENT_NODE,ownerDocument,parentNode);
        rectreateAttrs();
        tagName = owner.runJS(JSAccessors.getObjectPropertyAccessor(nodeAccessor,"tagName"));
        //if(nodeName.equals("FONT")){
            //setAttribute("color","#0000FF");
            //setAttribute("size","+2");
            //setAttribute("style","font-style: italic");
        //}
    }
    
    protected void rectreateAttrs(){
        int nAttributes = attributes.getAttrNumber();
        if(nAttributes > 0){
            attrs = new AttrWrapper[nAttributes];
            for(int i = 0; i < nAttributes; i++){
                attrs[i] = new AttrWrapper(owner,attributes.getItemAccessor(i),ownerDocument,this);
            }
        }
    }
    
    
    protected AttrWrapper getAtrrWrapperByName(String name){
        if(name == null || attrs == null || attrs.length < 1) return null;
        for(int i = 0; i < attrs.length; i++){
            AttrWrapper aw = attrs[i];
            if(aw.name == null) continue;
            if(aw.name.equalsIgnoreCase(name)){
                return aw;
            }
        }
        return null;
    }

    public void setNodeAccessor(String nodeAccessor){
        boolean oldOrphan = isOrphan();
        super.setNodeAccessor(nodeAccessor);
        if(oldOrphan && !isOrphan()){
            if(orphanAttributes!= null){
                orphanAttributes.clear();
                orphanAttributes = null;
            }
        }
    }

    
    public void setAttribute(String name,String value) throws DOMException{
        if(name == null || value == null) return;
        if(isOrphan()){
            if(orphanAttributes == null) orphanAttributes = new HashMap();
            orphanAttributes.put(name,value);
        }else{
            owner.runJS(nodeAccessor+".setAttribute(\""+name+"\",\""+value+"\")");
            AttrWrapper aw = getAtrrWrapperByName(name);
            if(aw != null){
                aw.value = value;
            }else{
                createAttributes();
                rectreateAttrs();
            }
        }
    }

    public String getCreateString(){
        if(!isOrphan()) return super.getCreateString();

        StringBuffer sb = new StringBuffer();
        sb.append("var newNode = window.document.createElement(\"");
        sb.append(tagName);
        sb.append("\");\n");
        if(orphanAttributes != null && orphanAttributes.size() > 0){
            java.util.Set keySet = orphanAttributes.keySet();
            java.util.Iterator it = keySet.iterator();
            while(it.hasNext()){
                String key = (String)it.next();
                String value = (String)orphanAttributes.get(key);
                sb.append("newNode.setAttribute(\"");
                sb.append(key);
                sb.append("\",\"");
                sb.append(value);
                sb.append("\");\n");
            }
        }
        return sb.toString();
    }

    protected void populateElementVector(Vector v,String name){
        if(v == null || name == null) return;
        if(children == null) return;
        for(int i = 0; i < children.getLength(); i++){
            NodeWrapper node = (NodeWrapper)children.item(i);
            if(node instanceof ElementWrapper){
                ElementWrapper element = (ElementWrapper)node;
                String elementTagName = element.getTagName();
                if(elementTagName != null){
                    if(elementTagName.equals(name)){
                        v.addElement(node);
                    }
                }
                ((ElementWrapper)node).populateElementVector(v,name);
            }
        }
    }

    public NodeList getElementsByTagName(String name){//???????
        if(children == null) return null;
        Vector nodes = new Vector();
        populateElementVector(nodes,name);
        if(nodes.size() < 1) return null;
        NodeWrapper []n = new NodeWrapper[nodes.size()];
        for(int i = 0; i < n.length; i++){
            n[i] = (NodeWrapper)nodes.elementAt(i);
        }
        return new NodeListWrapper(n);
    }

    public Attr getAttributeNode(String name){
        return getAtrrWrapperByName(name);
    }
    
    public String getAttribute(String name){
        return owner.runJS(nodeAccessor+".getAttribute(\""+name+"\")");
    }
 
    public boolean hasAttribute(String name){
        String ha = owner.runJS(nodeAccessor+".hasAttribute(\""+name+"\")");
        return ha.equalsIgnoreCase("true");
    }
   

    public String getTagName(){
        return tagName;
    }

    public String toString(){
        StringBuffer sb = new StringBuffer();
        if(attrs != null && attrs.length > 0){
            sb.append(" ATTRIBUTES\n");
            for(int i = 0; i < attrs.length; i++){
                sb.append(attrs[i].toString()+"\n");
            }
        }
        return "ELEMENT "+super.toString()+sb.toString()+"    ["+getNodeType()+"]";
    }

    public Attr setAttributeNode(Attr newAttr) throws DOMException{
        throw new RuntimeException("setAttributeNode isn't implemented");
    }
    
    public Attr removeAttributeNode(Attr oldAttr) throws DOMException{
        throw new RuntimeException("removeAttributeNode isn't implemented");
    }

    public void removeAttribute(String name) throws DOMException{
        throw new RuntimeException("removeAttribute isn't implemented");
    }
    
    public boolean hasAttributeNS(String namespaceURI, String localName){
        throw new RuntimeException("hasAttributeNS isn't implemented");
    }
    
    public String getAttributeNS(String namespaceURI, String localName){
        throw new RuntimeException("getAttributeNS isn't implemented");
    }

    public void setAttributeNS(String namespaceURI, String qualifiedName, String value) throws DOMException{
         throw new RuntimeException("setAttributeNS isn't implemented");
   }

    public void removeAttributeNS(String namespaceURI, String localName) throws DOMException{
        throw new RuntimeException("removeAttributeNS isn't implemented");
    }

    public Attr getAttributeNodeNS(String namespaceURI, String localName){
        throw new RuntimeException("getAttributeNodeNS isn't implemented");
    }

    public Attr setAttributeNodeNS(Attr newAttr) throws DOMException{
        throw new RuntimeException("setAttributeNodeNS isn't implemented");
    }

    public NodeList getElementsByTagNameNS(String namespaceURI, String localName){
        throw new RuntimeException("getElementsByTagNameNS isn't implemented");
    }
}
/*
    var rootElement = window.document.documentElement;
    var text = window.document.createText("dimatest");
    var element = window.document.createElement("FONT");
    element.setAttribute("color","00FF00");
    element.appendChild(text);
    rootElement.appendChild(element);
*/

