/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit.dom;
import org.dm.webkit.DMWebKitView;

public class JSAccessors{
    public static String getProperty(DMWebKitView owner,String objectAccessor,String property){
        return owner.runJS(getObjectPropertyAccessor(objectAccessor,property));
    }
    
    public static void setProperty(DMWebKitView owner,String objectAccessor,String property,String propertyValue){
        String jsString = getObjectPropertyAccessor(objectAccessor,property)+"=\""+propertyValue+"\"";
        owner.runJS(jsString);
    }
    
    public static String getObjectArrayAccessor(String objectAccessor,String arrayName,int index){
        return (objectAccessor+"."+arrayName+"["+index+"]");
    }
    
    public static String getObjectPropertyAccessor(String objectAccessor,String property){
        return objectAccessor+"."+property;
    }
    
    public static String getObjectMethodAccessor(String objectAccessor,String property){
        return objectAccessor+"."+property+"()";
    }
    
    public static String callMethod(DMWebKitView owner,String objectAccessor,String methodName,String []args){
        StringBuffer sb = new StringBuffer(objectAccessor+"."+methodName+"(");
        if(args != null && args.length > 0){
            for(int i = 0; i < args.length; i++){
                if(i > 0) sb.append(",");
                sb.append("\""+args[i]+"\"");
            }
        }
        sb.append(")");
        return owner.runJS(sb.toString());
    }
    
    public static int getArrayLength(DMWebKitView owner,String arrayAccessor){
        int retValue = 0;
        String str = getProperty(owner,arrayAccessor,"length");
        try{
            retValue = Integer.parseInt(str);
        }catch(Throwable t){
            retValue = 0;
        }
        return retValue;
    }
    
    public static short getNodeType(DMWebKitView owner,String nodeAccessor){
        short nodeType = NodeWrapper.UNDEFINED_NODE;
        String nodeTypeStr = JSAccessors.getProperty(owner,nodeAccessor,"nodeType");
        try{
            nodeType = Short.parseShort(nodeTypeStr);
        }catch(Throwable t){
            nodeType = NodeWrapper.UNDEFINED_NODE;
        }
        return nodeType;
    }
    
    public static int getIntFromString(String str){
        return getIntFromString(str,0);
    }
    
    public static int getIntFromString(String str,int defaultValue){
        int retValue = defaultValue;
        try{
            retValue = Integer.parseInt(str);
        }catch(Throwable t){
            retValue = defaultValue;
        }
        return retValue;
    }
}
